import React from 'react'
import ReactDOM from 'react-dom'
import PropTypes from 'prop-types'
import { setDefaultProps, classNames } from '../../utils/assist'
import withPreventScroll from '../../hoc/withPreventScroll'
import withHashChange from '../../hoc/withHashChange'
import { CSSTransition } from 'react-transition-group'
import './style/dialog.less'

const prefixCls = 'r-dialog'

class Dialog extends React.PureComponent {
  constructor(props) {
    super(props)
    this.state = {
      visible: props.visible,
      prevPropVisible: props.visible
    }
    this.container = props.portalTarget || this.portalTarget
    this.dialog = React.createRef()
    this.dialogWrap = React.createRef()
  }
  static getDerivedStateFromProps (nextProps, preState) {
    if (nextProps.visible !== preState.prevPropVisible) {
      return {
        visible: nextProps.visible,
        prevPropVisible: nextProps.visible
      }
    }
    return null
  }
  componentDidMount () {
    this.isMount = true
  }
  componentWillUnmount () {
    this.isMount = false
  }
  onEnter = () => {
    this.resetPos()
    this.props.withPreventScroll && this.props.withPreventScroll.init({ el: this.dialog.current })
  }
  onExited = () => {
    this.isMount = false
    this.props.fromInstance && ReactDOM.unmountComponentAtNode(this.props.portalTarget)
  }
  onCancel = (e) => {
    this.props.onCancel && this.props.onCancel.call(this, e)
  }
  onConfirm = (e) => {
    this.props.onConfirm && this.props.onConfirm.call(this, e)
  }
  onClose = (e) => {
    this.props.onClose && this.props.onClose.call(this, e)
  }
  remove = () => {
    this.isMount && this.setState({ visible: false })
  }
  resetPos () {
    const dialogWrap = this.dialogWrap.current
    const dWidth = dialogWrap.offsetWidth
    const dHeight = dialogWrap.offsetHeight
    const wWidth = window.innerWidth || document.documentElement.clientWidth || document.body.clientWidth
    const wHeight = window.innerHeight || document.documentElement.clientHeight || document.body.clientHeight

    if (dWidth > wWidth) dialogWrap.style.width = wWidth + 'px'
    if (dHeight > wHeight) dialogWrap.style.height = wHeight + 'px'

    switch (this.props.position.x) {
      case 'left':
        dialogWrap.style.left = '0px'
        break
      case 'center':
        dialogWrap.style.left = (wWidth - dWidth) / 2 + 'px'
        break
      case 'right':
        dialogWrap.style.right = '0px'
        break
      default:
        dialogWrap.style.left = this.props.position.x
    }

    switch (this.props.position.y) {
      case 'top':
        dialogWrap.style.top = '0px'
        break
      case 'center':
        dialogWrap.style.top = (wHeight - dHeight) / 2 + 'px'
        break
      case 'bottom':
        dialogWrap.style.bottom = '0px'
        break
      default:
        dialogWrap.style.top = this.props.position.y
    }
  }
  render () {
    const { showTitle, title, titleStyleObj, showCloseBtn, closeStyleObj, message, children, contentStyleObj, showCancelBtn, cancelBtnText, cancelBtnStyleObj, showConfirmBtn, confirmBtnText, confirmBtnStyleObj, cliperStyleObj, dlgStyleObj, className, from$dialog, cssTransitionClassName, cssTransitionTimeout, unmountOnExit } = this.props
    const wrapCls = [prefixCls, className, showCancelBtn && `${prefixCls}-cancel-btn-show`, showConfirmBtn && `${prefixCls}-confirm-btn-show`]
    const content = (
      <CSSTransition in={this.state.visible} appear classNames={cssTransitionClassName} timeout={cssTransitionTimeout} mountOnEnter={true} unmountOnExit={unmountOnExit} onEnter={this.onEnter} onExited={this.onExited}>
        <div className={classNames(wrapCls)} ref={this.dialog} style={cliperStyleObj}>
          <div className={`${prefixCls}-wrap`} ref={this.dialogWrap} style={dlgStyleObj}>
            {showTitle && <div className={`${prefixCls}-title`} style={titleStyleObj}>
              {title}
            </div>}
            {showCloseBtn && <div className={`${prefixCls}-close`} style={closeStyleObj} onClick={this.onClose}><svg width="11px" height="11px" viewBox="0 0 11 11" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlnsXlink="http://www.w3.org/1999/xlink">
              <g stroke="none" strokeWidth="1" fill="none" fillRule="evenodd">
                <g transform="translate(-249.000000, -10.000000)"
                  fill="#999999">
                  <path d="M254.50013,14.8683272 L249.762514,10.1306735 C249.588122,9.95644215 249.305423,9.95644215 249.130794,10.1306735 C248.956402,10.3052215 248.956402,10.5880359 249.130794,10.7624255 L253.868428,15.500023 L249.130795,20.2376113 C248.956402,20.4120016 248.956402,20.6948302 249.130795,20.8692073 C249.305425,21.0435976 249.588125,21.0435976 249.76228,20.8692073 L254.499971,16.1315614 L259.237665,20.8692176 C259.411819,21.0435941 259.694768,21.0435941 259.869384,20.8692176 C260.043539,20.694828 260.043539,20.4120004 259.869384,20.2376239 L255.131668,15.4998705 L259.869205,10.7623788 C260.043598,10.5880017 260.043598,10.3053314 259.869205,10.1307828 C259.694588,9.95640572 259.411875,9.95640572 259.23772,10.1307828 L254.50013,14.8683272 Z"></path>
                </g>
              </g>
            </svg></div>}
            {(message || children) && <div className={`${prefixCls}-content`} style={contentStyleObj}>
              {message} {children}
            </div>}
            { (showCancelBtn || showConfirmBtn) && <div className={`${prefixCls}-btn`}>
              {showCancelBtn && <div className={`${prefixCls}-cancel-btn`} style={cancelBtnStyleObj} onClick={this.onCancel}>
                {cancelBtnText}
              </div>}
              {showConfirmBtn && <div className={`${prefixCls}-confirm-btn`} style={confirmBtnStyleObj} onClick={this.onConfirm}>
                {confirmBtnText}
              </div>}
            </div>}
          </div>
        </div>
      </CSSTransition>
    )
    return this.container && !from$dialog ? ReactDOM.createPortal(content, this.container) : content
  }
}

setDefaultProps(Dialog, {
  // 标题
  title: {
    type: PropTypes.string,
    default: ''
  },
  showTitle: {
    type: PropTypes.bool,
    default: false
  },
  titleStyleObj: {
    type: PropTypes.object,
    default: {}
  },
  // closeBtn
  showCloseBtn: {
    type: PropTypes.bool,
    default: false
  },
  closeStyleObj: {
    type: PropTypes.object,
    default: {}
  },
  onClose: {
    type: PropTypes.func
  },
  // 内容
  message: {
    type: PropTypes.node,
    default: ''
  },
  contentStyleObj: {
    type: PropTypes.object
  },
  // cancelBtn
  cancelBtnText: {
    type: PropTypes.node,
    default: '取消'
  },
  showCancelBtn: {
    type: PropTypes.bool,
    default: true
  },
  cancelBtnStyleObj: {
    type: PropTypes.object,
    default: {}
  },
  onCancel: {
    type: PropTypes.func
  },
  // confirmBtn
  confirmBtnText: {
    type: PropTypes.node,
    default: '确定'
  },
  showConfirmBtn: {
    type: PropTypes.bool,
    default: true
  },
  confirmBtnStyleObj: {
    type: PropTypes.object,
    default: {}
  },
  onConfirm: {
    type: PropTypes.func
  },
  // 显示位置
  position: {
    type: PropTypes.object,
    default: { x: 'center', y: 'center' }
  },
  // hash变化时移除dialog
  removeDialogOnHashChange: {
    type: PropTypes.bool,
    default: true
  },
  // 自定义class
  className: {
    type: PropTypes.string,
    default: ''
  },
  // 可见
  visible: {
    type: PropTypes.bool,
    default: false
  },
  // cssTransition className
  cssTransitionClassName: {
    type: PropTypes.string,
    default: `${prefixCls}-fade`
  },
  // cssTransition timeout
  cssTransitionTimeout: {
    type: PropTypes.number,
    default: 300
  },
  // ummountOnExit 默认为false，表示当传入in的值为false值，CSSTransition中的元素不从dom中移除，传true时则会被移除
  unmountOnExit: {
    type: PropTypes.bool,
    default: false
  },
  // 遮罩层
  cliperStyleObj: {
    type: PropTypes.object
  },
  // dialog框
  dlgStyleObj: {
    type: PropTypes.object
  }
})

Dialog = withPreventScroll(Dialog)
Dialog = withHashChange(Dialog)



export default Dialog