import React, {memo} from 'react'
import PropTypes from 'prop-types'
import { setDefaultProps, classNames } from '../../utils/assist'
import withPreventScroll from '../../hoc/withPreventScroll'
import './style/flex-fixed.less'

const prefixCls = 'r-flexfixed'

class FlexFixed extends React.PureComponent {
  constructor(props) {
    super(props)
    this.state = {
      showFooter: false
    }
    this.flexFixedRef = React.createRef()
  }
  get wrapCls () {
    const { className } = this.props
    return classNames([
      prefixCls,
      className
    ])
  }
  get innerCls () {
    return `${prefixCls}-inner`
  }
  onMainScroll = (e) => {
    let main = e.currentTarget
    let scrollTop = main.scrollTop
    let {onScroll, onScrollDebounce} = this.props
    let direction = ''

    if (scrollTop > this.prevScrollTop) {
      direction = 'up'
    } else if (scrollTop < this.prevScrollTop) {
      direction = 'down'
    }

    if (e.target.timer) clearTimeout(e.target.timer)

    e.target.timer = setTimeout(() => {
      onScrollDebounce && onScrollDebounce({
        scrollTop: main.scrollTop,
        maxScrollTop: this.mainMaxScrollTop,
        direction
      })
    }, 150);
    onScroll && onScroll({
      scrollTop: main.scrollTop,
      maxScrollTop: this.mainMaxScrollTop,
      direction
    })
    this.prevScrollTop = scrollTop
  }
  onMainTouchStart = (e) => {
    let main = e.currentTarget
    this.mainMaxScrollTop = main.scrollHeight - main.offsetHeight
  }
  componentDidMount () {
    this.props.withPreventScroll && this.props.withPreventScroll.init({el: this.flexFixedRef.current})
  }
  hideFooterOnKeyboardPopup () {
    let doc = document.documentElement || document.body
    let docHeight = doc.clientHeight
  }
  render () {
    console.log('flexfixed mounted or updated')
    const { header, children, footer, extra } = this.props
    const {showFooter} = this.state
    return <div className={this.wrapCls} ref={this.flexFixedRef}>
      <div className={this.innerCls}>
        {header && <header>
          {header}
        </header>}
        <main className='scroll-area' onScroll={this.onMainScroll} onTouchStart={this.onMainTouchStart}>
          {children}
        </main>
        {footer && <footer style={{display : showFooter ? '' : 'none'}}>
          {footer}
        </footer>}
      </div>
      {extra}
    </div>
  }
}

setDefaultProps(FlexFixed, {
  className: {
    type: PropTypes.string
  },
  header: {
    type: PropTypes.node
  },
  footer: {
    type: PropTypes.node
  },
  extra: {
    type: PropTypes.node
  },
  onScroll: {
    type: PropTypes.func
  },
  onScrollDebounce: {
    type: PropTypes.func
  },
  hideFooterOnKeyboardShow: {
    type: PropTypes.bool,
    default: false
  }
})

export default withPreventScroll(FlexFixed)