import { memo, useEffect, useState, useRef } from "react"
import PropTypes from 'prop-types'
import { setDefaultProps, classNames } from '../../utils/assist'
import './style/switch.less'

const prefixCls = 'r-switch'

const Switch = ({ checked, disabled, checkedChildren, unCheckedChildren, onChange }) => {
  const [onOff, setOnOff] = useState(checked)
  const [maxLeft, setMaxLeft] = useState()

  const switchRef = useRef()
  const circleRef = useRef()

  console.log('Switch mounted or updated')

  useEffect(() => {
    setOnOff(checked)
  }, [checked])

  useEffect(() => {
    setMaxLeft(switchRef.current.offsetWidth - circleRef.current.offsetWidth - 3)
  }, [])

  const wrapCls = classNames([
    prefixCls,
    {
      [`${prefixCls}-checked`]: onOff
    }
  ])
  const innerCls = `${prefixCls}-inner`

  const circleStyle = {
    left: onOff ? maxLeft + 'px' : ''
  }

  const onSwitchClick = (e) => {
    if (disabled) return
    let tempOnOff = !onOff
    setOnOff(tempOnOff)
    onChange && onChange(tempOnOff)
  }

  return <div className={wrapCls} ref={switchRef} onClick={onSwitchClick}>
    <div className={innerCls}>
      <div className='open'>{checkedChildren}</div>
      <div className='close'>{unCheckedChildren}</div>
    </div>
    <div className='circle' ref={circleRef} style={circleStyle}></div>
  </div>
}

setDefaultProps(Switch, {
  checked: {
    type: PropTypes.bool,
    default: false
  },
  disabled: {
    type: PropTypes.bool
  },
  checkedChildren: {
    type: PropTypes.node
  },
  unCheckedChildren: {
    type: PropTypes.node
  },
  onChange: {
    type: PropTypes.func
  }
})

export default memo(Switch)